@@ -323,8 +323,7 @@ class SentencePieceFeaturizer(TextFeaturizer):
         self.upoints = None
         # vocab size
         self.num_classes = self.model.get_piece_size()
-        # create upoints
-        self.__init_upoints()
+        self.upoints = None
 
     def __init_upoints(self):
         text = [""]
@@ -451,6 +450,8 @@ class SentencePieceFeaturizer(TextFeaturizer):
         Returns:
             unicode code points transcript with dtype tf.int32 and shape [None]
         """
+        if self.upoints is None:
+            self.__init_upoints()
         with tf.name_scope("indices2upoints"):
             indices = self.normalize_indices(indices)
             upoints = tf.gather_nd(self.upoints, tf.expand_dims(indices, axis=-1))
