@@ -370,13 +370,12 @@ class BaseTester(BaseRunner):
             "greed_cer": ErrorRate(func=cer, name="test_greed_cer", dtype=tf.float32)
         }
 
-    def set_output_file(self):
+    def set_output_file(self, batch_size: int = 1):
+        if not batch_size: batch_size = self.config.batch_size
         if os.path.exists(self.output_file_path):
             with open(self.output_file_path, "r", encoding="utf-8") as out:
-                self.processed_records = get_num_batches(
-                    len(out.read().splitlines()) - 1,
-                    batch_size=1
-                )
+                self.processed_records = get_num_batches(len(out.read().splitlines()) - 1, batch_size=batch_size,
+                                                         drop_remainders=False)
         else:
             with open(self.output_file_path, "w") as out:
                 out.write("PATH\tGROUNDTRUTH\tGREEDY\tBEAMSEARCH\tBEAMSEARCHLM\n")
@@ -396,7 +395,7 @@ class BaseTester(BaseRunner):
         self.model = trained_model
 
     def run(self, test_dataset, batch_size=None):
-        self.set_output_file()
+        self.set_output_file(batch_size=batch_size)
         self.set_test_data_loader(test_dataset, batch_size=batch_size)
         self._test_epoch()
         self._finish()
